@@ -1,26 +1,25 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-
-import os
-import sys
 import json
-import pytest
+import os
 import shutil
+import sys
 import urllib.parse
-
 from binascii import hexlify
 
-import tornado
-from tornado.escape import url_escape
 import jupyter_core.paths
 import nbformat
+import pytest
+import tornado
+from tornado.escape import url_escape
 from traitlets.config import Config
 
 from jupyter_server.extension import serverextension
 from jupyter_server.serverapp import ServerApp
-from jupyter_server.utils import url_path_join, run_sync
 from jupyter_server.services.contents.filemanager import FileContentsManager
 from jupyter_server.services.contents.largefilemanager import LargeFileManager
+from jupyter_server.utils import run_sync
+from jupyter_server.utils import url_path_join
 
 
 # List of dependencies needed for this plugin.
@@ -33,12 +32,14 @@ pytest_plugins = [
 
 
 import asyncio
+
 if os.name == "nt" and sys.version_info >= (3, 7):
     asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
 
 
 # ============ Move to Jupyter Core =============
 
+
 def mkdir(tmp_path, *parts):
     path = tmp_path.joinpath(*parts)
     if not path.exists():
@@ -114,18 +115,15 @@ def jp_environ(
     monkeypatch.setenv("JUPYTER_CONFIG_DIR", str(jp_config_dir))
     monkeypatch.setenv("JUPYTER_DATA_DIR", str(jp_data_dir))
     monkeypatch.setenv("JUPYTER_RUNTIME_DIR", str(jp_runtime_dir))
-    monkeypatch.setattr(
-        jupyter_core.paths, "SYSTEM_JUPYTER_PATH", [str(jp_system_jupyter_path)]
-    )
+    monkeypatch.setattr(jupyter_core.paths, "SYSTEM_JUPYTER_PATH", [str(jp_system_jupyter_path)])
     monkeypatch.setattr(jupyter_core.paths, "ENV_JUPYTER_PATH", [str(jp_env_jupyter_path)])
-    monkeypatch.setattr(
-        jupyter_core.paths, "SYSTEM_CONFIG_PATH", [str(jp_system_config_path)]
-    )
+    monkeypatch.setattr(jupyter_core.paths, "SYSTEM_CONFIG_PATH", [str(jp_system_config_path)])
     monkeypatch.setattr(jupyter_core.paths, "ENV_CONFIG_PATH", [str(jp_env_config_path)])
 
 
 # ================= End: Move to Jupyter core ================
 
+
 @pytest.fixture
 def jp_server_config():
     """Allows tests to setup their specific configuration values. """
@@ -168,21 +166,21 @@ def jp_nbconvert_templates(jp_data_dir):
 
     # Get path to nbconvert template directory *before*
     # monkeypatching the paths env variable via the jp_environ fixture.
-    possible_paths = jupyter_core.paths.jupyter_path('nbconvert', 'templates')
+    possible_paths = jupyter_core.paths.jupyter_path("nbconvert", "templates")
     nbconvert_path = None
     for path in possible_paths:
         if os.path.exists(path):
             nbconvert_path = path
             break
 
-    nbconvert_target = jp_data_dir / 'nbconvert' / 'templates'
+    nbconvert_target = jp_data_dir / "nbconvert" / "templates"
 
     # copy nbconvert templates to new tmp data_dir.
     if nbconvert_path:
         shutil.copytree(nbconvert_path, str(nbconvert_target))
 
 
-@pytest.fixture(scope='function')
+@pytest.fixture(scope="function")
 def jp_configurable_serverapp(
     jp_nbconvert_templates,  # this fixture must preceed jp_environ
     jp_environ,
@@ -225,7 +223,7 @@ def jp_configurable_serverapp(
         token = hexlify(os.urandom(4)).decode("ascii")
         app = ServerApp.instance(
             # Set the log level to debug for testing purposes
-            log_level='DEBUG',
+            log_level="DEBUG",
             port=http_port,
             port_retries=0,
             open_browser=False,
@@ -264,10 +262,12 @@ def jp_ensure_app_fixture(request):
     """
     app_option = request.config.getoption("app_fixture")
     if app_option not in ["app", "jp_web_app"]:
-        raise Exception("jp_serverapp requires the `app-fixture` option "
-                        "to be set to 'jp_web_app`. Try rerunning the "
-                        "current tests with the option `--app-fixture "
-                        "jp_web_app`.")
+        raise Exception(
+            "jp_serverapp requires the `app-fixture` option "
+            "to be set to 'jp_web_app`. Try rerunning the "
+            "current tests with the option `--app-fixture "
+            "jp_web_app`."
+        )
     elif app_option == "app":
         # Manually set the app_fixture to `jp_web_app` if it's
         # not set already.
@@ -275,12 +275,7 @@ def jp_ensure_app_fixture(request):
 
 
 @pytest.fixture(scope="function")
-def jp_serverapp(
-    jp_ensure_app_fixture,
-    jp_server_config,
-    jp_argv,
-    jp_configurable_serverapp
-):
+def jp_serverapp(jp_ensure_app_fixture, jp_server_config, jp_argv, jp_configurable_serverapp):
     """Starts a Jupyter Server instance based on the established configuration values."""
     app = jp_configurable_serverapp(config=jp_server_config, argv=jp_argv)
     yield app
@@ -320,7 +315,12 @@ def jp_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_base_url):
             response = await jp_fetch("api", "spec.yaml")
             ...
     """
-    def client_fetch(*parts, headers={}, params={}, **kwargs):
+
+    def client_fetch(*parts, headers=None, params=None, **kwargs):
+        if not headers:
+            headers = {}
+        if not params:
+            params = {}
         # Handle URL strings
         path_url = url_escape(url_path_join(*parts), plus=False)
         base_path_url = url_path_join(jp_base_url, path_url)
@@ -329,9 +329,8 @@ def jp_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_base_url):
         # Add auth keys to header
         headers.update(jp_auth_header)
         # Make request.
-        return http_server_client.fetch(
-            url, headers=headers, request_timeout=20, **kwargs
-        )
+        return http_server_client.fetch(url, headers=headers, request_timeout=20, **kwargs)
+
     return client_fetch
 
 
@@ -362,45 +361,46 @@ def jp_ws_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_http_port,
             )
             ...
     """
-    def client_fetch(*parts, headers={}, params={}, **kwargs):
+
+    def client_fetch(*parts, headers=None, params=None, **kwargs):
+        if not headers:
+            headers = {}
+        if not params:
+            params = {}
         # Handle URL strings
         path_url = url_escape(url_path_join(*parts), plus=False)
         base_path_url = url_path_join(jp_base_url, path_url)
-        urlparts = urllib.parse.urlparse('ws://localhost:{}'.format(jp_http_port))
-        urlparts = urlparts._replace(
-            path=base_path_url,
-            query=urllib.parse.urlencode(params)
-        )
+        urlparts = urllib.parse.urlparse("ws://localhost:{}".format(jp_http_port))
+        urlparts = urlparts._replace(path=base_path_url, query=urllib.parse.urlencode(params))
         url = urlparts.geturl()
         # Add auth keys to header
         headers.update(jp_auth_header)
         # Make request.
-        req = tornado.httpclient.HTTPRequest(
-            url,
-            headers=jp_auth_header,
-            connect_timeout=120
-        )
+        req = tornado.httpclient.HTTPRequest(url, headers=headers, connect_timeout=120)
         return tornado.websocket.websocket_connect(req)
+
     return client_fetch
 
 
 some_resource = u"The very model of a modern major general"
 sample_kernel_json = {
-    'argv':['cat', '{connection_file}'],
-    'display_name': 'Test kernel',
+    "argv": ["cat", "{connection_file}"],
+    "display_name": "Test kernel",
 }
+
+
 @pytest.fixture
 def jp_kernelspecs(jp_data_dir):
     """Configures some sample kernelspecs in the Jupyter data directory."""
-    spec_names = ['sample', 'sample 2']
+    spec_names = ["sample", "sample 2"]
     for name in spec_names:
-        sample_kernel_dir = jp_data_dir.joinpath('kernels', name)
+        sample_kernel_dir = jp_data_dir.joinpath("kernels", name)
         sample_kernel_dir.mkdir(parents=True)
         # Create kernel json file
-        sample_kernel_file = sample_kernel_dir.joinpath('kernel.json')
+        sample_kernel_file = sample_kernel_dir.joinpath("kernel.json")
         sample_kernel_file.write_text(json.dumps(sample_kernel_json))
         # Create resources text
-        sample_kernel_resources = sample_kernel_dir.joinpath('resource.txt')
+        sample_kernel_resources = sample_kernel_dir.joinpath("resource.txt")
         sample_kernel_resources.write_text(some_resource)
 
 
@@ -419,10 +419,11 @@ def jp_large_contents_manager(tmp_path):
 @pytest.fixture
 def jp_create_notebook(jp_root_dir):
     """Creates a notebook in the test's home directory."""
+
     def inner(nbpath):
         nbpath = jp_root_dir.joinpath(nbpath)
         # Check that the notebook has the correct file extension.
-        if nbpath.suffix != '.ipynb':
+        if nbpath.suffix != ".ipynb":
             raise Exception("File extension for notebook must be .ipynb")
         # If the notebook path has a parent directory, make sure it's created.
         parent = nbpath.parent
@@ -431,6 +432,7 @@ def jp_create_notebook(jp_root_dir):
         nb = nbformat.v4.new_notebook()
         nbtext = nbformat.writes(nb, version=4)
         nbpath.write_text(nbtext)
+
     return inner
 
 
