@@ -1,10 +1,9 @@
+import asyncio
+import json
 import os
 import shutil
-import pytest
-import json
-import asyncio
-import sys
 
+import pytest
 from tornado.httpclient import HTTPClientError
 from traitlets.config import Config
 
@@ -15,12 +14,13 @@ from traitlets.config import Config
 def kill_all(jp_serverapp):
     async def _():
         await jp_serverapp.web_app.settings["terminal_manager"].kill_all()
+
     return _
 
 
 @pytest.fixture
 def terminal_path(tmp_path):
-    subdir = tmp_path.joinpath('terminal_path')
+    subdir = tmp_path.joinpath("terminal_path")
     subdir.mkdir()
 
     yield subdir
@@ -28,26 +28,29 @@ def terminal_path(tmp_path):
     shutil.rmtree(str(subdir), ignore_errors=True)
 
 
-CULL_TIMEOUT = 2
+CULL_TIMEOUT = 10
 CULL_INTERVAL = 3
 
 
 @pytest.fixture
 def jp_server_config():
-    return Config({
-        'ServerApp': {
-            'TerminalManager': {
-                'cull_inactive_timeout': CULL_TIMEOUT,
-                'cull_interval': CULL_INTERVAL
+    return Config(
+        {
+            "ServerApp": {
+                "TerminalManager": {
+                    "cull_inactive_timeout": CULL_TIMEOUT,
+                    "cull_interval": CULL_INTERVAL,
+                }
             }
         }
-    })
+    )
 
 
 async def test_no_terminals(jp_fetch):
     resp_list = await jp_fetch(
-        'api', 'terminals',
-        method='GET',
+        "api",
+        "terminals",
+        method="GET",
         allow_nonstandard_methods=True,
     )
 
@@ -58,16 +61,18 @@ async def test_no_terminals(jp_fetch):
 
 async def test_terminal_create(jp_fetch, kill_all):
     resp = await jp_fetch(
-        'api', 'terminals',
-        method='POST',
+        "api",
+        "terminals",
+        method="POST",
         allow_nonstandard_methods=True,
     )
     term = json.loads(resp.body.decode())
-    assert term['name'] == "1"
+    assert term["name"] == "1"
 
     resp_list = await jp_fetch(
-        'api', 'terminals',
-        method='GET',
+        "api",
+        "terminals",
+        method="GET",
         allow_nonstandard_methods=True,
     )
 
@@ -80,54 +85,56 @@ async def test_terminal_create(jp_fetch, kill_all):
 
 async def test_terminal_create_with_kwargs(jp_fetch, jp_ws_fetch, terminal_path, kill_all):
     resp_create = await jp_fetch(
-        'api', 'terminals',
-        method='POST',
-        body=json.dumps({'cwd': str(terminal_path)}),
+        "api",
+        "terminals",
+        method="POST",
+        body=json.dumps({"cwd": str(terminal_path)}),
         allow_nonstandard_methods=True,
     )
 
     data = json.loads(resp_create.body.decode())
-    term_name = data['name']
+    term_name = data["name"]
 
     resp_get = await jp_fetch(
-        'api', 'terminals', term_name,
-        method='GET',
+        "api",
+        "terminals",
+        term_name,
+        method="GET",
         allow_nonstandard_methods=True,
     )
 
     data = json.loads(resp_get.body.decode())
 
-    assert data['name'] == term_name
+    assert data["name"] == term_name
     await kill_all()
 
 
 async def test_terminal_create_with_cwd(jp_fetch, jp_ws_fetch, terminal_path):
     resp = await jp_fetch(
-        'api', 'terminals',
-        method='POST',
-        body=json.dumps({'cwd': str(terminal_path)}),
+        "api",
+        "terminals",
+        method="POST",
+        body=json.dumps({"cwd": str(terminal_path)}),
         allow_nonstandard_methods=True,
     )
 
     data = json.loads(resp.body.decode())
-    term_name = data['name']
+    term_name = data["name"]
 
-    ws = await jp_ws_fetch(
-        'terminals', 'websocket', term_name
-    )
+    ws = await jp_ws_fetch("terminals", "websocket", term_name)
 
-    ws.write_message(json.dumps(['stdin', 'pwd\r\n']))
+    ws.write_message(json.dumps(["stdin", "pwd\r\n"]))
 
-    message_stdout = ''
+    message_stdout = ""
     while True:
         try:
-            message = await asyncio.wait_for(ws.read_message(), timeout=1.0)
+            message = await asyncio.wait_for(ws.read_message(), timeout=5.0)
         except asyncio.TimeoutError:
             break
 
         message = json.loads(message)
 
-        if message[0] == 'stdout':
+        if message[0] == "stdout":
             message_stdout += message[1]
 
     ws.close()
@@ -139,7 +146,7 @@ async def test_culling_config(jp_server_config, jp_configurable_serverapp):
     terminal_mgr_config = jp_configurable_serverapp().config.ServerApp.TerminalManager
     assert terminal_mgr_config.cull_inactive_timeout == CULL_TIMEOUT
     assert terminal_mgr_config.cull_interval == CULL_INTERVAL
-    terminal_mgr_settings = jp_configurable_serverapp().web_app.settings['terminal_manager']
+    terminal_mgr_settings = jp_configurable_serverapp().web_app.settings["terminal_manager"]
     assert terminal_mgr_settings.cull_inactive_timeout == CULL_TIMEOUT
     assert terminal_mgr_settings.cull_interval == CULL_INTERVAL
 
@@ -147,20 +154,23 @@ async def test_culling_config(jp_server_config, jp_configurable_serverapp):
 async def test_culling(jp_server_config, jp_fetch):
     # POST request
     resp = await jp_fetch(
-        'api', 'terminals',
-        method='POST',
+        "api",
+        "terminals",
+        method="POST",
         allow_nonstandard_methods=True,
     )
     term = json.loads(resp.body.decode())
-    term_1 = term['name']
-    last_activity = term['last_activity']
+    term_1 = term["name"]
+    last_activity = term["last_activity"]
 
     culled = False
-    for i in range(10):  # Culling should occur in a few seconds
+    for i in range(CULL_TIMEOUT + CULL_INTERVAL):
         try:
             resp = await jp_fetch(
-                'api', 'terminals', term_1,
-                method='GET',
+                "api",
+                "terminals",
+                term_1,
+                method="GET",
                 allow_nonstandard_methods=True,
             )
         except HTTPClientError as e:
